GCC = gcc
AR = ar
ZYBOGCC = /remote/Xilinx/2015.4/SDK/2015.4/gnu/arm/lin/bin/arm-xilinx-eabi-gcc
ZYBOAR = /remote/Xilinx/2015.4/SDK/2015.4/gnu/arm/lin/bin/arm-xilinx-eabi-ar

INCDIR = $(TOP)/inc
OBJDIR = obj
ZYBOOBJDIR = obj-zybo
LIBDIR = $(TOP)/lib
ZYBOLIBDIR = $(TOP)/lib-zybo

SOURCES = $(wildcard *.c)
TESTSOURCES = $(wildcard test/*.c)
HEADERS = $(wildcard *.h)
INCLUDES = $(addprefix $(INCDIR)/, $(HEADERS))
OBJECTS = $(patsubst %.c, $(OBJDIR)/%.o, $(SOURCES))
ZYBOOBJECTS = $(patsubst %.c, $(ZYBOOBJDIR)/%.o, $(SOURCES))
TESTOBJECTS = $(patsubst $.c, %.o, $(TESTSOURCES))

TARGET = $(LIBDIR)/lib$(NAME).a
ZYBOTARGET = $(ZYBOLIBDIR)/lib$(NAME).a
TESTBIN = run_tests

.PHONY: default zybo all test clean

################
## User Targets
################

default: $(TARGET) $(INCLUDES)

zybo: $(ZYBOTARGET) $(INCLUDES)

all: default zybo

test: $(TESTBIN)
	./$(TESTBIN)

clean:
	rm -rf $(TARGET) $(ZYBOTARGET) $(INCLUDES) $(OBJDIR) $(ZYBOOBJDIR)

####################
## Internal Targets
####################

$(TARGET): $(OBJECTS) | $(LIBDIR)
	$(AR) rcs $@ $^

$(ZYBOTARGET): $(ZYBOOBJECTS) | $(ZYBOLIBDIR)
	$(ZYBOAR) rcs $@ $^

$(OBJDIR)/%.o : %.c | $(OBJDIR) $(INCDIR)
	$(GCC) -c -g -o $@ $< -I$(INCDIR)

$(ZYBOOBJDIR)/%.o : %.c | $(ZYBOOBJDIR) $(INCDIR)
	$(ZYBOGCC) -c -g -o $@ $< -I$(INCDIR)

$(INCDIR)/%.h : %.h | $(INCDIR)
	cp $^ $(INCDIR)

$(INCDIR):
	mkdir $(INCDIR)

$(OBJDIR):
	mkdir $(OBJDIR)

$(LIBDIR):
	mkdir $(LIBDIR)

$(ZYBOOBJDIR):
	mkdir $(ZYBOOBJDIR)

$(ZYBOLIBDIR):
	mkdir $(ZYBOLIBDIR)


$(TESTBIN): $(TESTOBJECTS) $(OBJECTS)
	$(GCC) -o $(TESTBIN) $^ -I$(INCDIR) -L$(LIBDIR) $(REQLIBS)
